using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace AIOUSBNet
{
    public class AIOUSB         // Prototype AIOUSB.dll data and functions in managed C# for .Net -- Watch for char arrays, UNICODE v ANSI, and strings
    {
        #region Constants for simplifying use of Device Index
        public const uint diNone = 0xFFFFFFFF;    // "-1"
        public const uint diFirst = 0xFFFFFFFE;    // "-2"  First board (lowest DeviceIndex)
        public const uint diOnly = 0xFFFFFFFD;    // "-3"  One and only board
        #endregion
        #region Constants for Win32 Error codes
        public const uint ERROR_SUCCESS = 0;
        #endregion
        #region Constants, Misc
        public const int MAXNAMESIZE = 32;
        #endregion
        # region Constants related to AIOUSB_ClearFIFO() and similar
        public const uint TIME_METHOD_NOW = 0;
        public const uint TIME_METHOD_WAIT_INPUT_ENABLE = 86;
        public const uint TIME_METHOD_NOW_AND_ABORT = 5;
        public const uint TIME_METHOD_WHEN_INPUT_ENABLE = 1;
        #endregion
        #region General Board Utility Functions

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint GetDevices();

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint QueryDeviceInfo(uint DeviceIndex, ref uint pPID, ref int pNameSize, IntPtr Name, ref uint DIOBytes, ref uint Counters);

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint QueryDeviceInfo(uint DeviceIndex, out uint pPID, out int pNameSize, StringBuilder DEVName, out uint DIOBytes, out uint Counters);

        public static uint QueryDeviceInformation(uint DeviceIndex, out uint pPID, out int pNameSize, out string Name, out uint DIOBytes, out uint Counters)
        {
            uint Status;
            StringBuilder strNameIn = new StringBuilder( AIOUSB.MAXNAMESIZE + 1 );
            int NameSize;
            Status = AIOUSB.QueryDeviceInfo(DeviceIndex, out pPID, out NameSize, strNameIn, out DIOBytes, out Counters);
            if (Status == AIOUSB.ERROR_SUCCESS)
            {
                //Name = strNameIn.ToString(0, NameSize);
                Name = "";
                 pNameSize = NameSize;
            }else
            {
                 Name = "";
                pNameSize = 0;
            }
            return Status;
        }
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint AIOUSB_ReloadDeviceLinks();

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ResolveDeviceIndex(uint DeviceIndex);

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint AIOUSB_CloseDevice(uint DeviceIndex);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ClearDevices();

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint CustomEEPROMRead(uint DeviceIndex, uint StartAddress, ref uint DataSize, [Out] byte[] Data);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint CustomEEPROMWrite(uint DeviceIndex, uint StartAddress, uint DataSize, [In] byte[] Data);

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint GetDeviceByEEPROMByte(byte Data);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint GetDeviceByEEPROMData(uint StartAddress, uint DataSize, [In] byte[] pData);

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint GetDeviceSerialNumber(uint DeviceIndex, out ulong pSerialNumber);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint AIOUSB_ClearFIFO(uint DeviceIndex, uint TimeMethod);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint AIOUSB_QuerySimplePNPData(uint DeviceIndex, ref byte[] pPNPData, ref uint pPNPDataBytes);
        #endregion
        #region DIO (Digital I/O, Including FET, Relay, and Isolated Inputs) Specific Functions

        // DIO_Configure is overloaded for your convenience
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DIO_Configure(uint DeviceIndex, byte Tristate, ref uint OutMask, ref uint Data);
        
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DIO_Configure(uint DeviceIndex, byte Tristate, ref short OutMask, [In] byte[] Data);
        
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DIO_Configure(uint DeviceIndex, byte Tristate, [In] byte[] OutMask, [In] byte[] Data);
        
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DIO_Configure(uint DeviceIndex, byte Tristate, ref byte OutMask, ref byte[] Data);
        
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DIO_Configure(uint deviceIndex, byte bTristateAll, ref uint outMask, ref byte[] Data);

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DIO_Configure(uint deviceIndex, byte bTristateAll, IntPtr outMask, IntPtr Data);

        public static uint DIO_Configure(uint DeviceIndex, byte bTristateAll, int OutsLength, byte[] Outs, int DataLength, byte[] Data)
        {
            IntPtr OutsPtr = Marshal.AllocHGlobal(OutsLength);
            IntPtr DataPtr = Marshal.AllocHGlobal(DataLength);
            Marshal.Copy(Outs, 0, OutsPtr, OutsLength);
            Marshal.Copy(Data, 0, DataPtr, DataLength);
            uint Status = DIO_Configure(DeviceIndex, bTristateAll, OutsPtr, DataPtr);

            return Status;
        }



        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DIO_ConfigureMasked(uint DeviceIndex, [In] byte[] pOuts, [In] byte[] pOutsMask, [In] byte[] pData, [In] byte[] pDataMask, [In] byte[] pTristates, [In] byte[] pTristatesMask);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DIO_ConfigurationQuery(uint DeviceIndex, [Out] byte[] pOutMask, [Out] byte[] pTristateMask);



        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]

        // DIO_WriteAll is overloaded for your convenience
        public static extern uint DIO_WriteAll(uint DeviceIndex, ref uint Data);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DIO_WriteAll(uint DeviceIndex, [In] byte[] Data);

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DIO_Write8(uint DeviceIndex, uint ByteIndex, byte Data);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DIO_Write1(uint DeviceIndex, uint BitIndex, byte Data);

        // DIO_ReadAll is overloaded for your convenience


        // [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)] public static extern UInt32 DIO_ReadAll(UInt32 DeviceIndex, ref UInt32 Data);

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        private static extern uint DIO_ReadAll(uint DeviceIndex, IntPtr Data);

        public static uint DIO_ReadAll(uint DeviceIndex, int dataLength, byte[] Data)
        {
            IntPtr outPtr = Marshal.AllocHGlobal(dataLength);
            uint Status = DIO_ReadAll(DeviceIndex, outPtr);
            Marshal.Copy(outPtr, Data, 0, dataLength);
            return Status;
        }
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DIO_ReadAll(uint DeviceIndex, ref uint Data);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DIO_ReadAll(uint DeviceIndex, [Out] byte[] Data);

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DIO_Read8(uint DeviceIndex, uint ByteIndex, ref byte Data);
        
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DIO_Read1(uint DeviceIndex, uint BitIndex, out byte Data); // edit by John 6/18/21
        
        #endregion
        #region USB-DIO-16H Family-specific (streaming, fast) functions

        #endregion
        #region ADC (Analog Input (Analog-to-Digital)) Specific Functions
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_GetScanV(uint DeviceIndex, [Out] double[] pBuf);
        #region USB-AIx and DPK-AIx Family-Specific Functions
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_SetConfig(uint DeviceIndex, [In] byte[] pConfigBuf, ref uint ConfigBufSize);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_GetConfig(uint DeviceIndex, [Out] byte[] pConfigBuf, ref uint ConfigBufSize);

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_Initialize(uint DeviceIndex, [In] byte pConfigBuf, ref uint pConfigBufSize, [In] char[] CalFileName); // wrapper for SetConfig AND SetCal, nothing more.
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_QueryCal(uint DeviceIndex);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_SetCalAndSave(uint DeviceIndex, [In] char[] CalFileName, [In] char[] SaveCalFileName);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_SetCal(uint DeviceIndex, [In] char[] CalFileName);
        public static uint ADC_SetCal(uint DeviceIndex, string CalFileName)
        {
            uint NameSize = Convert.ToUInt32(CalFileName.Length + 1);  // This is the only func in dll that takes a null terminator, so add 1 character for it
            char[] charName = new char[NameSize];
            CalFileName.CopyTo(0, charName, 0, CalFileName.Length);
            return ADC_SetCal(DeviceIndex, charName);
        }
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_BulkAcquire(uint DeviceIndex, uint BufSize, [Out] ushort[] Buf);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_BulkPoll(uint DeviceIndex, [Out] uint BytesLeft);

        // Continuous mode for USB-AIx and DPK-AIx:

        // Callback function delegate for use as function pointer parameter passed to AIOUSB.DLL:
        // Declare delegate -- defines required signature:
        // Original from Pascal: procedure ADCallback(pBuf: PWord; BufSize, Flags, Context: LongWord); cdecl;
        // Must be Cdecl not default stdcall to match dll:
        // We need to use Intptr for the buffer:
        // Application probably needs to use GCHandle.Alloc(foo), where "foo" is the callback function name, to prevent garbage collection of the callback function
        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate void ADCallback(IntPtr pBuf, uint BufSize, uint Flags, uint Context);

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_BulkContinuousCallbackStart(uint DeviceIndex, uint BufSize, uint BaseBufCount, uint Context, ADCallback pCallback);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_FullStartRing(uint DeviceIndex, [Out] byte[] pConfigBuf, ref uint ConfigBufSize, [ Out] char[] CalFileName, ref double pCounterHz, uint RingBufferSize, uint PacketsPerBlock);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_ReadData(uint DeviceIndex, [Out] byte[] pConfigBuf, ref uint ScansToRead, ref double[] pData, ref double Timeout);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_ReadData(uint DeviceIndex, [Out] byte[] pConfigBuf, ref uint ScansToRead, ref double[][] pDataScans, ref double Timeout);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_BulkContinuousEnd(uint DeviceIndex, [Out] uint pIOStatus);
        #endregion
        #endregion
        #region DAC (Analog Output (Digital-to-Analog)) Specific Functions

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DACDirect(uint DeviceIndex, ushort Channel, ushort Counts);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DACMultiDirect(uint DeviceIndex, [In] ushort[] pDACData, uint DACDataCount);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DACSetBoardRange(uint DeviceIndex, uint RangeCode);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DACSetChannelCal(uint DeviceIndex, uint Channel, [In] char[] CalFileName);
        public static uint DACSetChannelCal(uint DeviceIndex, uint Channel, string CalFileName)
        {
            uint NameSize = Convert.ToUInt32(CalFileName.Length);
            char[] charName = new char[NameSize];
            CalFileName.CopyTo(0, charName, 0, CalFileName.Length);
            return DACSetChannelCal(DeviceIndex, Channel, charName);
        }

        #region DAC Functions for USB-DA12-8A only
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DACOutputProcess(uint DeviceIndex, ref double ClockHz, uint NumSamples, [In] ushort[] SampleData);

        // More DAC Functions for USB-DA12-8A only (advanced, rare)
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DACOutputOpen(uint DeviceIndex, ref double ClockHZ);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DACOutputCloseNoEnd(uint DeviceIndex, uint bWait);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DACOutputFrameRaw(uint DeviceIndex, uint FramePoints, [In] ushort FrameData);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DACOutputSetCount(uint DeviceIndex, uint NewCount);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DACOutputSetInterlock(uint DeviceIndex, uint bInterlock);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DACOutputStart(uint DeviceIndex);
        #endregion
        #endregion
        #region CTR (Counter/Timer (8254)) Specific Functions
        // more 8254 functions (rare, advanced)
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint CTR_8254Load(uint DeviceIndex, uint BlockIndex, uint CounterIndex, ushort LoadValue);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint CTR_8254Mode(uint DeviceIndex, uint BlockIndex, uint CounterIndex, uint Mode);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint CTR_8254ModeLoad(uint DeviceIndex, uint BlockIndex, uint CounterIndex, uint Mode, ushort LoadValue);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint CTR_8254Read(uint DeviceIndex, uint BlockIndex, uint CounterIndex, [Out] ushort ReadValue);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint CTR_8254ReadStatus(uint DeviceIndex, uint BlockIndex, uint CounterIndex, [Out] ushort ReadValue, [Out] byte Status);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint CTR_8254ReadModeLoad(uint DeviceIndex, uint BlockIndex, uint CounterIndex, uint Mode, ushort LoadValue, [Out] ushort ReadValue);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint CTR_StartOutputFreq(uint DeviceIndex, uint BlockIndex, ref double Hz);
        #region USB-CTR-15 Specific Functions
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint CTR_8254ReadAll(uint DeviceIndex, [Out] ushort[] pData);

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint CTR_8254SelectGate(uint DeviceIndex, uint GateIndex);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint CTR_8254ReadLatched(uint DeviceIndex, [Out] ushort[] pData);

        // Legacy functions that were for a custom Application
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint CTR_SetWaitGates(uint DeviceIndex, byte A, byte B);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint CTR_EndWaitGates(uint DeviceIndex);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint CTR_WaitForGate(uint DeviceIndex, byte GateIndex, ref ushort pContent);

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint CTR_StartMeasuringPulseWidth(uint DeviceIndex);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint CTR_StopMeasuringPulseWidth(uint DeviceIndex);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint CTR_GetPulseWidthMeasurement(uint DeviceIndex, uint BlockIndex, uint CounterIndex, [Out] ushort pReadValue);
        #endregion
        #endregion
        #region These functions are reserved for factory use only, or are deprecated
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint GenericVendorRead(uint DeviceIndex, byte Request, ushort Value, ushort Index, ref uint pDataSize, byte[] pData);

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint GenericVendorWrite(uint DeviceIndex, byte Request, ushort Value, ushort Index, uint DataSize, ref byte[] pData);

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DIO_SPI_Read(uint DeviceIndex, uint Address, byte Reg, [Out] byte[] pValue);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DIO_SPI_Write(uint DeviceIndex, uint Address, byte Reg, byte Value);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_Initialize(uint DeviceIndex, [In, Out] byte[] pConfigBuf, ref uint ConfigBufSize, [In, Out] char[] CalFileName);  // Unicode
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_GetImmediate(uint DeviceIndex, uint Channel, out ushort pBuf);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DACOutputClose(uint DeviceIndex, uint bWait);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint AIOUSB_FlashRead(uint DeviceIndex, uint StartAddress, [In, Out] uint DataSize, ref byte[] Data);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint AIOUSB_FlashWrite(uint DeviceIndex, uint StartAddress, uint DataSize, [Out] byte[] Data);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint AIOUSB_FlashEraseSector(uint DeviceIndex, uint Sector);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DACOutputFrame(uint DeviceIndex, uint FramePoints, [In, Out] ushort pFrameData);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DAC_CSA_SetRangeLimits(uint DeviceIndex, [In, Out] byte[] pData);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint DAC_CSA_ClearRangeLimits(uint DeviceIndex);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_InitFastITScanV(uint DeviceIndex);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_ResetFastITScanV(uint DeviceIndex);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_SetFastITScanVChannels(uint DeviceIndex, uint NewChannels);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_GetFastITScanV(uint DeviceIndex, [In, Out] double pBuf);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_GetITScanV(uint DeviceIndex, [In, Out] double pBuf);

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_CSA_InitFastScanV(uint DeviceIndex);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_CSA_ResetFastScanV(uint DeviceIndex);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_CSA_GetFastScanV(uint DeviceIndex, [In, Out] double[] pBuf);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_RangeAll(uint DeviceIndex, ref byte GainCodes, uint bDifferential);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_Range1(uint DeviceIndex, uint ADChannel, byte GainCode, uint bDifferential);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_ADMode(uint DeviceIndex, byte TriggerMode, byte CalMode);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_SetScanLimits(uint DeviceIndex, uint StartChannel, uint EndChannel);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_SetOversample(uint DeviceIndex, byte Oversample);

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_GetCalRefV(uint DeviceIndex, uint CalRefIndex, out double pRef);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_BulkContinuousCallbackStartClocked(uint DeviceIndex, uint BufSize, uint BaseBufCount, uint Context, ADCallback pCallback, ref double Hz);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint GetDeviceBySerialNumber(ulong SerialNumber);



        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint AIOUSB_GetStreamStatus(uint DeviceIndex, out uint Status);

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint AIOUSB_SetStreamingBlockSize(uint DeviceIndex, uint BlockSize);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint GetDeviceUniqueStr(uint DeviceIndex, ref uint pIIDSize, [In, Out] char pIID);

        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_GetChannelV(uint DeviceIndex, uint ChannelIndex, out double pBuf); // If you're taking more than one channel of data, ADC_GetScanV() is faster!
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_GetScan(uint DeviceIndex, [In, Out] ushort[] pBuf);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_BulkMode(uint DeviceIndex, uint BulkMode);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern uint ADC_BulkContinuousRingStart(uint DeviceIndex, uint RingBufferSize, uint PacketsPerBlock);

        //__declspec(dllimport) unsigned long DIO_CSA_DoSync(unsigned long DeviceIndex, double * BaseRateHz, double * DurAms, double * DurBms, double * DurCms);
        //__declspec(dllimport) unsigned long DIO_CSA_DebounceSet(unsigned long DeviceIndex, unsigned long DebounceCounts);
        //__declspec(dllimport) unsigned long DIO_CSA_DebounceReadAll(unsigned long DeviceIndex, void *pData);
        //__declspec(dllimport) unsigned long DACDIO_WriteAll(unsigned long DeviceIndex, unsigned short *pDACCounts, unsigned char *pDIOData);
        #endregion
    }
}
