﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Sharp7;
using AIOUSBNet;
using System.Diagnostics;

namespace Sample1
{
    static class Program
    {
        static uint DetectADCandDIODeviceIndex()
        {
            _ = AIOUSB.AIOUSB_ReloadDeviceLinks(); // redetects devices; generally just for convenience; do this first as DeviceIndices can change
            gv.diDIO = AIOUSB.diNone;
            uint diMask = AIOUSB.GetDevices();
            for (uint DeviceIndex = 0; DeviceIndex < 32; DeviceIndex++)
            {
                if ((diMask & (1 << (int)(DeviceIndex))) > 0)
                {
                    uint pPID;
                    _ = AIOUSB.QueryDeviceInformation(DeviceIndex, out pPID, out _, out _, out _, out _);
                    // detect ADC devices
                    if (((pPID >= 0x8040) && (pPID < 0x805F))
                         || ((pPID >= 0x8140) && (pPID < 0x815F)))
                    {
                        gv.diADC = DeviceIndex;
                        continue;
                    }
                    // detect USB-IIRO-16 devices
                    if ((pPID >= 0x8010) && (pPID <= 0x801F))
                    {
                        gv.diDIO = (uint)DeviceIndex;
                        continue;
                    }
                }
            }
            if ((gv.diADC != AIOUSB.diNone) && (gv.diDIO != AIOUSB.diNone))
                return 1;
            return 0; /* ERROR_SUCCESS */
        }

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            //****************************************************************************************
            //*********** FORT WAYNE INDIANA TESTER TYPE "CONSOLE" OR "HEADLINER" ********************
            //****************************************************************************************

            /* //select either test type
            gv.FW_TESTER_TYPE = "HEADLINER";
            /*/ 
            gv.FW_TESTER_TYPE = "CONSOLE";
            //*/
            gv.SOFTWARE_VERSION = "FW Software Ver. (1.0.0) 7/1/21";

            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);

            if (DetectADCandDIODeviceIndex() != 0 /* ERROR_SUCCES*/ )
            {
                //handle either or both ADC/DIO devices not found
            }

            while (!gv.bCloseProgram)
            {
                if (gv.FW_TESTER_TYPE.Contains("CONSOLE"))
                {
                    if (gv.ConsoleTestMainForm != null) gv.ConsoleTestMainForm.Close();
                    gv.ConsoleTestMainForm = new ConsoleConnectTestForm3();

                    if (gv.DiagnosticForm != null) gv.DiagnosticForm.Close();
                    gv.DiagnosticForm = new DiagnosticForm();

                    if (gv.ConsoleTestResultForm != null) gv.ConsoleTestResultForm.Close();
                    gv.ConsoleTestResultForm = new ConsoleTestResultsForm();

                    if (gv.CSNEntryForm != null) gv.CSNEntryForm.Close();
                    gv.CSNEntryForm = new CSNEntryForm();

                    Application.Run(gv.ConsoleTestMainForm); // 3rd Console Test Connectors
                }

                if (gv.FW_TESTER_TYPE.Contains("HEADLINER"))
                {
                    if (gv.HeadlinerConnectTestForm != null) gv.HeadlinerConnectTestForm.Close();
                    gv.HeadlinerConnectTestForm = new HeadlinerConnectTestForm();// 3rd Console Test Connectors

                    if (gv.DiagnosticForm != null) gv.DiagnosticForm.Close();
                    gv.DiagnosticForm = new DiagnosticForm();// 5th  Analog Test Validation

                    if (gv.HeadlinerTestResultsOptionsForm != null) gv.HeadlinerTestResultsOptionsForm.Close();
                    gv.HeadlinerTestResultsOptionsForm = new HeadlinerTestResultsOptionsForm();// 2nd Console Test Option Selection / Test Results

                    if (gv.CSNEntryForm != null) gv.CSNEntryForm.Close();
                    gv.CSNEntryForm = new CSNEntryForm();// 1st VIN Keypad

                    Application.Run(gv.HeadlinerConnectTestForm); 
                }
            }
        }
    }
}
